package fr.aminelouveaau.cartes.service;

import fr.aminelouveaau.cartes.model.Card;
import fr.aminelouveaau.cartes.model.Number;
import fr.aminelouveaau.cartes.model.Shape;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.*;

@Service
public class GameService {
    @Value("${cards.hand.number}")
    private Integer cardHandNumber;

    private List<Shape> shapeOrder;
    private List<Number> numberOrder;

    private Stack<Card> deck = new Stack<>();
    private Comparator<Card> comparator;

    @PostConstruct
    public void init() {
        generateOrders();
        createDeck();
    }

    public List<Shape> getShapeOrder() {
        return shapeOrder;
    }

    public List<Number> getNumberOrder() {
        return numberOrder;
    }

    public List<Card> getDeck() {
        return deck;
    }

    private void createDeck() {
        Arrays.stream(Shape.values()).forEach(shape -> {
            Arrays.stream(Number.values()).forEach(number -> {
                deck.push(new Card(shape, number));
            });
        });
        Collections.shuffle(deck);
    }

    private void generateOrders() {
        shapeOrder = Arrays.asList(Shape.values());
        Collections.shuffle(shapeOrder);
        numberOrder = Arrays.asList(Number.values());
        Collections.shuffle(numberOrder);

        comparator = Comparator.comparing((Card c) -> shapeOrder.indexOf(c.getShape()))
                .thenComparing(c -> numberOrder.indexOf(c.getNumber()));
    }

}
